# Copyright (c) 2019 Renata Hodovan, Akos Kiss.
#
# Licensed under the BSD 3-Clause License
# <LICENSE.rst or https://opensource.org/licenses/BSD-3-Clause>.
# This file may not be copied, modified, or distributed except
# according to those terms.

import os
import pytest

from collections import OrderedDict

import xson


val_tuplekey = {(1, 2): 3, 4: 5}
exp_tuplekey_skipkeys = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx"><json:number name="4">5</json:number></json:object>
'''

val_circular_list = []
val_circular_list.append(val_circular_list)

val_circular_dict = {}
val_circular_dict[None] = val_circular_dict

val_nan = float('nan')
exp_nan_allownan = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:number xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">NaN</json:number>
'''

val_inf = float('inf')
exp_inf_allownan = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:number xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">Infinity</json:number>
'''

val_neginf = float('-inf')
exp_neginf_allownan = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:number xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">-Infinity</json:number>
'''

val_dict_list = {1: [2]}
exp_dict_list_compact = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx"><json:array name="1"><json:number>2</json:number></json:array></json:object>
'''
exp_dict_list_newline = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
<json:array name="1">
<json:number>2</json:number>
</json:array>
</json:object>
'''
exp_dict_list_indent = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
  <json:array name="1">
    <json:number>2</json:number>
  </json:array>
</json:object>
'''

val_list_dict = [{1: 2}]
exp_list_dict_compact = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:array xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx"><json:object><json:number name="1">2</json:number></json:object></json:array>
'''
exp_list_dict_newline = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:array xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
<json:object>
<json:number name="1">2</json:number>
</json:object>
</json:array>
'''
exp_list_dict_indent = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:array xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
  <json:object>
    <json:number name="1">2</json:number>
  </json:object>
</json:array>
'''

val_object = object()

val_tuple = (1, 2)
exp_tuple_default = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:array xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx"><json:number>1</json:number><json:number>2</json:number></json:array>
'''

def tuple_default(val):
    if isinstance(val, tuple):
        return list(val)
    raise TypeError

val_ordereddict = OrderedDict([(4, 3), (2, 1)])
exp_ordereddict = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx"><json:number name="4">3</json:number><json:number name="2">1</json:number></json:object>
'''
exp_ordereddict_sortkeys = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx"><json:number name="2">1</json:number><json:number name="4">3</json:number></json:object>
'''

val_ordereddict_mixedkeys = OrderedDict([('a', 'b'), (1, 2)])
exp_ordereddict_mixedkeys = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx"><json:string name="a">b</json:string><json:number name="1">2</json:number></json:object>
'''

@pytest.mark.parametrize('val, kw, exp', [
    # skipkeys (default: False)
    (val_tuplekey, {}, TypeError),
    (val_tuplekey, {'skipkeys': False}, TypeError),
    (val_tuplekey, {'skipkeys': True}, exp_tuplekey_skipkeys),
    # check_circular (default: True)
    (val_circular_list, {}, ValueError),
    (val_circular_list, {'check_circular': True}, ValueError),
    (val_circular_list, {'check_circular': False}, RuntimeError),
    (val_circular_dict, {}, ValueError),
    (val_circular_dict, {'check_circular': True}, ValueError),
    (val_circular_dict, {'check_circular': False}, RuntimeError),
    # allow_nan (default: True)
    (val_nan, {}, exp_nan_allownan),
    (val_nan, {'allow_nan': True}, exp_nan_allownan),
    (val_nan, {'allow_nan': False}, ValueError),
    (val_inf, {}, exp_inf_allownan),
    (val_inf, {'allow_nan': True}, exp_inf_allownan),
    (val_inf, {'allow_nan': False}, ValueError),
    (val_neginf, {}, exp_neginf_allownan),
    (val_neginf, {'allow_nan': True}, exp_neginf_allownan),
    (val_neginf, {'allow_nan': False}, ValueError),
    # indent (default: None)
    (val_dict_list, {}, exp_dict_list_compact),
    (val_dict_list, {'indent': None}, exp_dict_list_compact),
    (val_dict_list, {'indent': -1}, exp_dict_list_newline),
    (val_dict_list, {'indent': 0}, exp_dict_list_newline),
    (val_dict_list, {'indent': ''}, exp_dict_list_newline),
    (val_dict_list, {'indent': 2}, exp_dict_list_indent),
    (val_dict_list, {'indent': '  '}, exp_dict_list_indent),
    (val_list_dict, {}, exp_list_dict_compact),
    (val_list_dict, {'indent': None}, exp_list_dict_compact),
    (val_list_dict, {'indent': -1}, exp_list_dict_newline),
    (val_list_dict, {'indent': 0}, exp_list_dict_newline),
    (val_list_dict, {'indent': ''}, exp_list_dict_newline),
    (val_list_dict, {'indent': 2}, exp_list_dict_indent),
    (val_list_dict, {'indent': '  '}, exp_list_dict_indent),
    # default (default: None)
    (val_object, {}, TypeError),
    (val_object, {'default': None}, TypeError),
    (val_object, {'default': tuple_default}, TypeError),
    (val_tuple, {}, TypeError),
    (val_tuple, {'default': None}, TypeError),
    (val_tuple, {'default': tuple_default}, exp_tuple_default),
    # sort_keys (default: False)
    (val_ordereddict, {}, exp_ordereddict),
    (val_ordereddict, {'sort_keys': False}, exp_ordereddict),
    (val_ordereddict, {'sort_keys': True}, exp_ordereddict_sortkeys),
    (val_ordereddict_mixedkeys, {}, exp_ordereddict_mixedkeys),
    (val_ordereddict_mixedkeys, {'sort_keys': False}, exp_ordereddict_mixedkeys),
    (val_ordereddict_mixedkeys, {'sort_keys': True}, TypeError),
])
def test_dump(val, kw, exp, tmpdir):
    def _dumps():
        return xson.dumps(val, **kw)

    def _dump():
        tmpfn = os.path.join(str(tmpdir), 'tmp.jsonx')
        with open(tmpfn, 'w') as tmpf:
            xson.dump(val, tmpf, **kw)
        with open(tmpfn, 'r') as tmpf:
            return tmpf.read()

    for dump in (_dumps, _dump):
        if isinstance(exp, type):
            with pytest.raises(exp):
                dump()
        else:
            out = dump()
            assert out.strip() == exp.strip()



# Copyright (c) 2019 Renata Hodovan, Akos Kiss.
#
# Licensed under the BSD 3-Clause License
# <LICENSE.rst or https://opensource.org/licenses/BSD-3-Clause>.
# This file may not be copied, modified, or distributed except
# according to those terms.

import json
import pytest

import xson


@pytest.mark.parametrize('value', [
    # null (None)
    None,
    # boolean (bool)
    True,
    False,
    # number (int, float; except NaN, which cannot be cross-validated as NaN != NaN)
    0,
    1,
    -1,
    42,
    3.14,
    -1.28,
    float('inf'),
    float('-inf'),
    # string (str)
    '',
    'foo',
    # array (list)
    [],
    [ None, True, False, 0, 1, -1, 42, 3.14, -1.28, float('inf'), float('-inf'), '', 'foo', [], {} ],
    # object (dict)
    {},
    { None: None },
    { True: True, False: False },
    { 0: 0, 1: 1, -1: -1, 42: 42, 3.14: 3.14, -1.28: -1.28, float('inf'): float('inf'), float('-inf'): float('-inf') },
    { '': None, 'a': True, 'b': False, 'c': 0, 'd': 1, 'e': -1, 'foo': 42, 'g': 3.14, 'h': -1.28, 'i': float('inf'), 'j': float('-inf'), 'k': '', 'l': 'foo', 'm': [], 'n': {} },
])
def test_cross_validate(value):
    xson_repr = xson.dumps(value)
    xson_value = xson.loads(xson_repr)

    json_repr = json.dumps(value)
    json_value = json.loads(json_repr)

    assert xson_value == json_value



# Copyright (c) 2019 Renata Hodovan, Akos Kiss.
#
# Licensed under the BSD 3-Clause License
# <LICENSE.rst or https://opensource.org/licenses/BSD-3-Clause>.
# This file may not be copied, modified, or distributed except
# according to those terms.

import pkg_resources


__version__ = pkg_resources.get_distribution(__package__).version

JSONX_PREFIX = 'json'
JSONX_NS_URI = 'http://www.ibm.com/xmlns/prod/2009/jsonx'



# Copyright (c) 2019 Renata Hodovan, Akos Kiss.
#
# Licensed under the BSD 3-Clause License
# <LICENSE.rst or https://opensource.org/licenses/BSD-3-Clause>.
# This file may not be copied, modified, or distributed except
# according to those terms.

from io import StringIO
from math import isinf, isnan
from xml.sax import make_parser
from xml.sax.handler import ContentHandler, ErrorHandler, feature_namespaces

from .pkgdata import JSONX_NS_URI


class JSONxElement(object):
    def __init__(self, localname, key, value):
        self.localname = localname
        self.key = key
        self.value = value


class JSONxHandler(ContentHandler, ErrorHandler):

    def __init__(self, object_hook=None, parse_float=None, parse_int=None, parse_constant=None, object_pairs_hook=None):
        super().__init__()

        self._object_hook = object_hook
        self._parse_float = parse_float
        self._parse_int = parse_int
        self._parse_constant = parse_constant
        self._object_pairs_hook = object_pairs_hook

        self.stack = [JSONxElement('root', None, None)]

    def startElement(self, name, attrs):
        self._expect(False, 'unsupported non-namespaced element %s' % name)

    def endElement(self, name):
        assert False, 'SAX parser must ensure that non-namespaced end of element does not occur'

    def startElementNS(self, name, qname, attrs):
        uri, localname = name
        self._expect(uri == JSONX_NS_URI, 'unsupported namespace URI %s' % uri)

        self._expect(self.stack[-1].localname in ('root', 'object', 'array'), '%s element cannot contain other elements' % self.stack[-1].localname)

        key = attrs[(None, 'name')] if (None, 'name') in attrs else None
        if self.stack[-1].localname == 'object':
            self._expect(key is not None, 'element within an object element must have a name attribute')

        if localname == 'object':
            self.stack.append(JSONxElement(localname, key, list()))
        elif localname == 'array':
            self.stack.append(JSONxElement(localname, key, list()))
        elif localname in ('string', 'number', 'boolean'):
            self.stack.append(JSONxElement(localname, key, StringIO()))
        elif localname == 'null':
            self.stack.append(JSONxElement(localname, key, None))
        else:
            self._expect(False, 'unsupported element %s' % localname)

    def endElementNS(self, name, qname):
        uri, localname = name
        assert uri == JSONX_NS_URI, 'SAX parser must ensure that namespace URIs of start and end of elements match'

        element = self.stack.pop()
        assert element.localname == localname, 'SAX parser must ensure that localnames of start and end of elements match'

        value = element.value
        if localname == 'object':
            if self._object_pairs_hook:
                value = self._object_pairs_hook(value)
            else:
                value = dict(value)
                if self._object_hook:
                    value = self._object_hook(value)
        elif localname == 'string':
            value = value.getvalue()
        elif localname == 'number':
            value = value.getvalue()
            try:
                value_default = int(value)
                value_parser = self._parse_int
            except ValueError:
                try:
                    value_default = float(value)
                    value_parser = self._parse_float if not isnan(value_default) and not isinf(value_default) else self._parse_constant
                except ValueError:
                    self._expect(False, 'number element must contain text content in floating point format')
            value = value_parser(value) if value_parser else value_default
        elif localname == 'boolean':
            value = value.getvalue()
            self._expect(value in ('true', 'false'), 'boolean element must contain either true or false text content')
            value = value == 'true'

        container = self.stack[-1]
        if container.localname == 'root':
            container.value = value
        elif container.localname == 'object':
            container.value.append((element.key, value))
        elif container.localname == 'array':
            container.value.append(value)
        else:
            assert False, 'unexpected container element %s' % container.localname

    def characters(self, content):
        element = self.stack[-1]
        if isinstance(element.value, StringIO):
            element.value.write(content)
        else:
            self._expect(content.isspace(), '%s element must not have non-whitespace character content %s' % (element.localname, content))

    def error(self, exception):
        self._expect(False, exception.getMessage())

    fatalError = error

    def warning(self, exception):
        # Suppress warnings.
        pass

    def _expect(self, expr, msg):
        if not expr:
            raise ValueError('%s [line %s, column %s]' % (msg, self._locator.getLineNumber(), self._locator.getColumnNumber()))


def load(fp, *, object_hook=None, parse_float=None, parse_int=None, parse_constant=None, object_pairs_hook=None):
    """
    Deserialize a JSONx file to a Python object.

    :param fp: File-like object to be deserialized.
    :param object_hook: If specified, it must be a function that will be called
        with the result of any object decoded (a :class:`dict`), and its return
        value will be used instead. (Default: ``None``)
    :param object_pairs_hook: If specified, it must be a function that will be
        called with the result of any object decoded into an ordered list of
        pairs, and its return value will be used instead. If ``object_hook`` is
        also specified, ``object_pairs_hook`` takes priority. (Default:
        ``None``)
    :param parse_float: If specified, it must be a function that will be called
        with the string of every float to be decoded. (Default: :class:`float`)
    :param parse_int: If specified, it must be a function that will be called
        with the string of every int to be decoded. (Default: :class:`int`)
    :param parse_constant: If specified, it must be a function that will be
        called with one of the following strings: ``'-Infinity'``,
        ``'Infinity'``, or ``'NaN'``. (Default: :class:`float`)
    :return: The value deserialized.
    :raises ValueError: If the data being deserialized is not a valid JSONx
        document.
    """

    handler = JSONxHandler(object_hook=object_hook, parse_float=parse_float, parse_int=parse_int, parse_constant=parse_constant, object_pairs_hook=object_pairs_hook)

    parser = make_parser()
    parser.setContentHandler(handler)
    parser.setErrorHandler(handler)
    parser.setFeature(feature_namespaces, True)
    parser.parse(fp)

    assert len(handler.stack) == 1 and handler.stack[0].localname == 'root'
    return handler.stack[0].value


def loads(s, *, object_hook=None, parse_float=None, parse_int=None, parse_constant=None, object_pairs_hook=None):
    """
    Deserialize a JSONx string to a Python object.

    :param str s: String to be deserialized.

    The keyword arguments have the same meaning as in :func:`load`.
    """

    return load(StringIO(s), object_hook=object_hook, parse_float=parse_float, parse_int=parse_int, parse_constant=parse_constant, object_pairs_hook=object_pairs_hook)



# Copyright (c) 2019-2020 Renata Hodovan, Akos Kiss.
#
# Licensed under the BSD 3-Clause License
# <LICENSE.rst or https://opensource.org/licenses/BSD-3-Clause>.
# This file may not be copied, modified, or distributed except
# according to those terms.

from io import StringIO
from math import isinf, isnan
from xml.sax.saxutils import XMLGenerator
from xml.sax.xmlreader import AttributesNSImpl

from .pkgdata import JSONX_NS_URI, JSONX_PREFIX


def dump(obj, fp, *, skipkeys=False, check_circular=True, allow_nan=True, indent=None, default=None, sort_keys=False):
    """
    Serialize a value to a file in JSONx format.

    :param obj: Value to be serialized.
    :param fp: File-like object to write JSONx :class:`str` to.
    :param bool skipkeys: If true, then dictionary keys that are not of a basic
        type (:class:`str`, :class:`int`, :class:`float`, :class:`bool`,
        ``None``) will be skipped. Otherwise, a :exc:`TypeError` is raised.
        (Default: ``False``)
    :param bool check_circular: If false, then the circular reference check for
        container types will be skipped. Otherwise, a :exc:`ValueError` is
        raised. (Default: ``True``)
    :param bool allow_nan: If false, then it will be a :exc:`ValueError` to
        serialize out-of-range float values (``nan``, ``inf``, ``-inf``).
        Otherwise, their JavaScript equivalents (``NaN``, ``Infinity``,
        ``-Infinity``) will be used. (Default: ``True``)
    :param indent: If a positive integer, then JSON array elements and object
        members will be pretty-printed with that many spaces per level. If a
        string, that string is used to indent each level. An indent level of 0,
        negative, or ``""`` will only insert newlines. ``None`` selects the most
        compact representation. (Default: ``None``)
    :type indent: int or str
    :param default: If specified, it must be a function that gets called for
        objects that can’t otherwise be serialized, and it must return an
        encodable version of the object. If not specified, :exc:`TypeError` is
        raised. (Default: ``None``)
    :param bool sort_keys: If true, then the output of dictionaries will be
        sorted by key. (Default: ``False``)
    """

    def _attrs(name=None):
        return AttributesNSImpl({(None, 'name'): name} if name is not None else {}, None)

    def _str(value):
        if value is None:
            return 'null'
        if isinstance(value, bool):
            return 'true' if value else 'false'
        if isinstance(value, float):
            if isnan(value):
                return 'NaN'
            if isinf(value):
                return 'Infinity' if value > 0 else '-Infinity'
        return str(value)

    def _dump(value, name=None):
        nonlocal stack, level
        if indent is not None:
            gen.ignorableWhitespace(indent * level)

        if isinstance(value, dict):
            if check_circular:
                if id(value) in stack:
                    raise ValueError('container has circular reference')
                stack.add(id(value))

            gen.startElementNS((JSONX_NS_URI, 'object'), None, attrs=_attrs(name))

            if value:
                if indent is not None:
                    gen.ignorableWhitespace('\n')
                level += 1

                for k, v in sorted(value.items(), key=lambda kv: kv[0]) if sort_keys else value.items():
                    if k is not None and not isinstance(k, (str, int, float, bool)):
                        if not skipkeys:
                            raise TypeError('dictionary key is not of a basic type: %r' % (k,))
                        continue
                    _dump(v, name=_str(k))

                level -= 1
                if indent is not None:
                    gen.ignorableWhitespace(indent * level)

            gen.endElementNS((JSONX_NS_URI, 'object'), None)

            if check_circular:
                stack.remove(id(value))

        elif isinstance(value, list):
            if check_circular:
                if id(value) in stack:
                    raise ValueError('container has circular reference')
                stack.add(id(value))

            gen.startElementNS((JSONX_NS_URI, 'array'), None, attrs=_attrs(name))

            if value:
                if indent is not None:
                    gen.ignorableWhitespace('\n')
                level += 1

                for v in value:
                    _dump(v)

                level -= 1
                if indent is not None:
                    gen.ignorableWhitespace(indent * level)

            gen.endElementNS((JSONX_NS_URI, 'array'), None)

            if check_circular:
                stack.remove(id(value))

        elif isinstance(value, str):
            gen.startElementNS((JSONX_NS_URI, 'string'), None, attrs=_attrs(name))
            gen.characters(value)
            gen.endElementNS((JSONX_NS_URI, 'string'), None)

        elif isinstance(value, bool):
            gen.startElementNS((JSONX_NS_URI, 'boolean'), None, attrs=_attrs(name))
            gen.characters(_str(value))
            gen.endElementNS((JSONX_NS_URI, 'boolean'), None)

        elif isinstance(value, (int, float)):
            if not allow_nan and (isinf(value) or isnan(value)):
                raise ValueError('float value is out of range: %r' % value)

            gen.startElementNS((JSONX_NS_URI, 'number'), None, attrs=_attrs(name))
            gen.characters(_str(value))
            gen.endElementNS((JSONX_NS_URI, 'number'), None)

        elif value is None:
            gen.startElementNS((JSONX_NS_URI, 'null'), None, attrs=_attrs(name))
            gen.endElementNS((JSONX_NS_URI, 'null'), None)

        elif default:
            _dump(default(value), name=name)

        else:
            raise TypeError('cannot serialize object: %r' % (value,))

        if indent is not None:
            gen.ignorableWhitespace('\n')

    stack = set()
    level = 0
    if indent is not None and isinstance(indent, int):
        indent = ' ' * indent if indent > 0 else ''

    gen = XMLGenerator(fp, encoding='UTF-8', short_empty_elements=True)
    gen.startDocument()
    gen.startPrefixMapping(JSONX_PREFIX, JSONX_NS_URI)
    _dump(obj)
    gen.endPrefixMapping(JSONX_PREFIX)
    gen.endDocument()


def dumps(obj, *, skipkeys=False, check_circular=True, allow_nan=True, indent=None, default=None, sort_keys=False):
    """
    Serialize a value to a string in JSONx format.

    The arguments have the same meaning as in :func:`dump`.

    :return: JSONx string.
    :rtype: str
    """

    s = StringIO()
    dump(obj, s, skipkeys=skipkeys, check_circular=check_circular, allow_nan=allow_nan, indent=indent, default=default, sort_keys=sort_keys)
    return s.getvalue()



# Copyright (c) 2019-2020 Renata Hodovan, Akos Kiss.
#
# Licensed under the BSD 3-Clause License
# <LICENSE.rst or https://opensource.org/licenses/BSD-3-Clause>.
# This file may not be copied, modified, or distributed except
# according to those terms.

from setuptools import find_packages, setup


setup(
    name='xson',
    packages=find_packages(),
    url='https://github.com/renatahodovan/xson',
    license='BSD',
    author='Renata Hodovan, Akos Kiss',
    author_email='hodovan@inf.u-szeged.hu, akiss@inf.u-szeged.hu',
    description='XSON: XML Encoding for JSON',
    long_description=open('README.rst').read(),
    zip_safe=False,
    include_package_data=True,
    setup_requires=['setuptools_scm'],
    use_scm_version=True,
    install_requires=[
        'setuptools',
    ],
    extras_require={
        'docs': [
            'sphinx',
            'sphinx_rtd_theme',
        ]
    },
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Utilities',
    ],
)



# Copyright (c) 2019 Renata Hodovan, Akos Kiss.
#
# Licensed under the BSD 3-Clause License
# <LICENSE.rst or https://opensource.org/licenses/BSD-3-Clause>.
# This file may not be copied, modified, or distributed except
# according to those terms.

import os
import pytest

from math import isnan

import xson


inp_tuple = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
    <json:array name="$tuple">
        <json:number>1</json:number>
        <json:number>2</json:number>
    </json:array>
</json:object>
'''
exp_tuple_dict = {'$tuple': [1, 2]}
exp_tuple = (1, 2)
exp_list = [1, 2]

def tuple_object_hook(obj):
    if '$tuple' in obj:
        return tuple(obj['$tuple'])
    return obj

def tuple_list_object_pairs_hook(pairs):
    obj = dict(pairs)
    if '$tuple' in obj:
        return list(obj['$tuple'])
    return obj

inp_one = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:number xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">1</json:number>
'''
exp_one_str = '$int: 1'

def parse_int_str(s):
    return '$int: %s' % s

inp_pi = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:number xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">3.14</json:number>
'''
exp_pi_str = '$float: 3.14'

def parse_float_str(s):
    return '$float: %s' % s

inp_nan = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:number xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">NaN</json:number>
'''
exp_nan_str = '$constant: NaN'

inp_inf = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:number xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">Infinity</json:number>
'''
exp_inf_str = '$constant: Infinity'

inp_neginf = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:number xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">-Infinity</json:number>
'''
exp_neginf_str = '$constant: -Infinity'

def parse_constant_str(s):
    return '$constant: %s' % s

@pytest.mark.parametrize('inp, kw, exp', [
    # object_hook (default: None)
    (inp_tuple, {}, exp_tuple_dict),
    (inp_tuple, {'object_hook': None}, exp_tuple_dict),
    (inp_tuple, {'object_hook': tuple_object_hook}, exp_tuple),
    # object_pairs_hook (default: None)
    (inp_tuple, {'object_pairs_hook': None}, exp_tuple_dict),
    (inp_tuple, {'object_pairs_hook': tuple_list_object_pairs_hook}, exp_list),
    (inp_tuple, {'object_pairs_hook': tuple_list_object_pairs_hook, 'object_hook': tuple_object_hook}, exp_list),
    # parse_int (default: None/int)
    (inp_one, {}, int(1)),
    (inp_one, {'parse_int': None}, int(1)),
    (inp_one, {'parse_int': parse_int_str}, exp_one_str),
    # parse_float (default: None/float)
    (inp_pi, {}, float(3.14)),
    (inp_pi, {'parse_float': None}, float(3.14)),
    (inp_pi, {'parse_float': parse_float_str}, exp_pi_str),
    # parse_constant (default: None/float)
    (inp_nan, {}, float('nan')),
    (inp_nan, {'parse_constant': None}, float('nan')),
    (inp_nan, {'parse_constant': parse_constant_str}, exp_nan_str),
    (inp_inf, {}, float('inf')),
    (inp_inf, {'parse_constant': None}, float('inf')),
    (inp_inf, {'parse_constant': parse_constant_str}, exp_inf_str),
    (inp_neginf, {}, float('-inf')),
    (inp_neginf, {'parse_constant': None}, float('-inf')),
    (inp_neginf, {'parse_constant': parse_constant_str}, exp_neginf_str),
])
def test_load(inp, kw, exp, tmpdir):
    def _loads():
        return xson.loads(inp.strip(), **kw)

    def _load():
        tmpfn = os.path.join(str(tmpdir), 'tmp.jsonx')
        with open(tmpfn, 'w') as tmpf:
            tmpf.write(inp.strip())
        with open(tmpfn, 'r') as tmpf:
            return xson.load(tmpf, **kw)

    for load in (_loads, _load):
        if isinstance(exp, type):
            with pytest.raises(exp):
                load()
        else:
            val = load()
            if isinstance(val, float) and isnan(val):
                assert isinstance(exp, float) and isnan(exp)
            else:
                assert val == exp



# Copyright (c) 2019 Renata Hodovan, Akos Kiss.
#
# Licensed under the BSD 3-Clause License
# <LICENSE.rst or https://opensource.org/licenses/BSD-3-Clause>.
# This file may not be copied, modified, or distributed except
# according to those terms.

import json
import pytest

import xson


# Test cases taken from:
#   Muschett, B., Salz, R., and Schenker, M., "JSONx, an XML Encoding for JSON",
#   Internet-Draft draft-rsalz-jsonx-00, May 2011,
#   <https://tools.ietf.org/html/draft-rsalz-jsonx-00>.
#
# NOTE:
# - Several inputs have been changed by wrapping them in curly braces to make
#   them valid JSON. The expected outputs have been changed accordingly to have
#   <json:object> as the outmost element.
# - Several outputs have been changed to include XML output declaration as well
#   as the JSONx namespace declaration.
# - Several outputs have been edited for whitespace.
# - The extended example input has been fixed to make it valid JSON (had both
#   missing and extra commas). The output has been sorted by object property
#   names to stabilize the test case.

object_json = '{ "Ticker" : "IBM" }'
object_xson = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
    <json:string name="Ticker">IBM</json:string>
</json:object>
'''

array_json = '''
{
    "phoneNumbers": [
        "212 555-1111",
        "212 555-2222"
    ]
}
'''
array_xson = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
    <json:array name="phoneNumbers">
        <json:string>212 555-1111</json:string>
        <json:string>212 555-2222</json:string>
    </json:array>
</json:object>
'''

boolean_json = '{ "remote": false }'
boolean_xson = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
    <json:boolean name="remote">false</json:boolean>
</json:object>
'''

string_json = '{ "name": "John Smith" }'
string_xson = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
    <json:string name="name">John Smith</json:string>
</json:object>
'''

number_json = '{ "height": 62.4 }'
number_xson = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
    <json:number name="height">62.4</json:number>
</json:object>
'''

null_json = '{ "additionalInfo": null }'
null_xson = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
    <json:null name="additionalInfo"/>
</json:object>
'''

example_json = '''
{
    "name": "John Smith",
    "address": {
        "streetAddress": "21 2nd Street",
        "city": "New York",
        "state": "NY",
        "postalCode": 10021
    },
    "phoneNumbers": [
        "212 555-1111",
        "212 555-2222"
    ],
    "additionalInfo": null,
    "remote": false,
    "height": 62.4,
    "ficoScore": "> 640"
}
'''
example_xson = '''
<?xml version="1.0" encoding="UTF-8"?>
<json:object xmlns:json="http://www.ibm.com/xmlns/prod/2009/jsonx">
    <json:null name="additionalInfo"/>
    <json:object name="address">
        <json:string name="city">New York</json:string>
        <json:number name="postalCode">10021</json:number>
        <json:string name="state">NY</json:string>
        <json:string name="streetAddress">21 2nd Street</json:string>
    </json:object>
    <json:string name="ficoScore">&gt; 640</json:string>
    <json:number name="height">62.4</json:number>
    <json:string name="name">John Smith</json:string>
    <json:array name="phoneNumbers">
        <json:string>212 555-1111</json:string>
        <json:string>212 555-2222</json:string>
    </json:array>
    <json:boolean name="remote">false</json:boolean>
</json:object>
'''

@pytest.mark.parametrize('json_inp, xson_exp', [
    (object_json, object_xson),
    (array_json, array_xson),
    (boolean_json, boolean_xson),
    (string_json, string_xson),
    (number_json, number_xson),
    (null_json, null_xson),
    (example_json, example_xson),
])
def test_internet_draft(json_inp, xson_exp):
    json_val = json.loads(json_inp)
    xson_out = xson.dumps(json_val, indent=4, sort_keys=True)

    assert xson_out.strip() == xson_exp.strip()

    xson_val = xson.loads(xson_out)

    assert xson_val == json_val
